@@ -4,24 +4,16 @@
 #include <asm/div64.h>
 #include "omfs.h"
 
-static int nibblemap[] = {4, 3, 3, 2, 3, 2, 2, 1, 3, 2, 2, 1, 2, 1, 1, 0};
-
 unsigned long omfs_count_free(struct super_block *sb)
 {
-	unsigned int i, j, count = 0;
-	unsigned char *map;
+	unsigned int i;
 	unsigned long sum = 0;
 	struct omfs_sb_info *sbi = OMFS_SB(sb);
+	int nbits = sb->s_blocksize * 8;
 
 	for (i = 0; i < sbi->s_imap_size; i++)
-	{
-		map = (unsigned char *) sbi->s_imap[i];
-		for (j=0; j < sb->s_blocksize && 
-				count + j < sbi->s_num_blocks/8; j++)
-			sum += nibblemap[map[j] & 0xf] +
-				nibblemap[(map[j] >> 4) & 0xf];
-		count += sb->s_blocksize;
-	}
+		sum += nbits - bitmap_weight(sbi->s_imap[i], nbits);
+
 	return sum;
 }
 
@@ -30,70 +22,66 @@ unsigned long omfs_count_free(struct super_block *sb)
  *  It handles the case where a run might spill over a buffer.
  *  Called with bitmap lock.
  */
-static int count_run(unsigned long **addr, int nbits, 
+static int count_run(unsigned long **addr, int nbits,
 		int addrlen, int bit, int max)
 {
 	int count = 0;
 	int x;
 
-	for (; addrlen > 0; addrlen--, addr++)
-	{
+	for (; addrlen > 0; addrlen--, addr++) {
 		x = find_next_bit(*addr, nbits, bit);
 		count += x - bit;
 
 		if (x < nbits || count > max)
-			return min(count,max);
+			return min(count, max);
 
 		bit = 0;
 	}
-	return min(count,max);
+	return min(count, max);
 }
 
 /*
  * Sets or clears the run of count bits starting with bit.
  * Called with bitmap lock.
  */
-static int set_run(struct super_block *sb, int map, 
+static int set_run(struct super_block *sb, int map,
 		int nbits, int bit, int count, int set)
 {
 	int i;
+	int err;
 	struct buffer_head *bh;
 	struct omfs_sb_info *sbi = OMFS_SB(sb);
 
+ 	err = -ENOMEM;
 	bh = sb_bread(sb, clus_to_blk(sbi, sbi->s_bitmap_ino) + map);
 	if (!bh)
-		goto nomem;
+		goto out;
 
-	for (i = 0; i < count; i++, bit++)
-	{
-		if (bit >= nbits)
-		{
+	for (i = 0; i < count; i++, bit++) {
+		if (bit >= nbits) {
 			bit = 0;
 			map++;
 
 			mark_buffer_dirty(bh);
 			brelse(bh);
-			bh = sb_bread(sb, 
+			bh = sb_bread(sb,
 				clus_to_blk(sbi, sbi->s_bitmap_ino) + map);
 			if (!bh)
-				goto nomem;
+				goto out;
 		}
-		if (set)
-		{
+		if (set) {
 			set_bit(bit, sbi->s_imap[map]);
-			set_bit(bit, (long*) bh->b_data);
-		}
-		else
-		{
+			set_bit(bit, (long *) bh->b_data);
+		} else {
 			clear_bit(bit, sbi->s_imap[map]);
-			clear_bit(bit, (long*) bh->b_data);
+			clear_bit(bit, (long *) bh->b_data);
 		}
 	}
 	mark_buffer_dirty(bh);
 	brelse(bh);
-	return 0;
-nomem:
-	return -ENOMEM;
+	err = 0;
+out:
+	return err;
 }
 
 /*
@@ -105,7 +93,7 @@ int omfs_allocate_block(struct super_block *sb, u64 block)
 	struct omfs_sb_info *sbi = OMFS_SB(sb);
 	int bits_per_entry = 8 * sb->s_blocksize;
 	int map, bit;
-	int ret = 1;
+	int ret = 0;
 	u64 tmp;
 
 	tmp = block;
@@ -113,25 +101,19 @@ int omfs_allocate_block(struct super_block *sb, u64 block)
 	map = tmp;
 
 	mutex_lock(&sbi->s_bitmap_lock);
-	if (map >= sbi->s_imap_size || 
-	    test_and_set_bit(bit, sbi->s_imap[map]))
-	{
-		ret = 0;
+	if (map >= sbi->s_imap_size || test_and_set_bit(bit, sbi->s_imap[map]))
 		goto out;
-	}
 
-	if (sbi->s_bitmap_ino > 0)
-	{
+	if (sbi->s_bitmap_ino > 0) {
 		bh = sb_bread(sb, clus_to_blk(sbi, sbi->s_bitmap_ino) + map);
 		if (!bh)
-		{
-			ret = 0;
 			goto out;
-		}
-		set_bit(bit, (long*) bh->b_data);
+
+		set_bit(bit, (long *) bh->b_data);
 		mark_buffer_dirty(bh);
 		brelse(bh);
 	}
+	ret = 1;
 out:
 	mutex_unlock(&sbi->s_bitmap_lock);
 	return ret;
@@ -139,13 +121,13 @@ out:
 
 
 /*
- *  Tries to allocate a set of blocks.	The request size depends on the 
- *  type: for inodes, we must allocate sbi->s_mirrors blocks, and for file 
+ *  Tries to allocate a set of blocks.	The request size depends on the
+ *  type: for inodes, we must allocate sbi->s_mirrors blocks, and for file
  *  blocks, we try to allocate sbi->s_clustersize, but can always get away
  *  with just one block.
  */
-int omfs_allocate_range(struct super_block *sb, 
-			int min_request, 
+int omfs_allocate_range(struct super_block *sb,
+			int min_request,
 			int max_request,
 			u64 *return_block,
 			int *return_size)
@@ -156,18 +138,16 @@ int omfs_allocate_range(struct super_block *sb,
 	int i, run, bit;
 
 	mutex_lock(&sbi->s_bitmap_lock);
-	for (i=0; i < sbi->s_imap_size; i++)
-	{
+	for (i = 0; i < sbi->s_imap_size; i++) {
 		bit = 0;
-		while (bit < bits_per_entry)
-		{
+		while (bit < bits_per_entry) {
 			bit = find_next_zero_bit(sbi->s_imap[i], bits_per_entry,
 				bit);
 
 			if (bit == bits_per_entry)
 				break;
 
-			run = count_run(&sbi->s_imap[i], bits_per_entry, 
+			run = count_run(&sbi->s_imap[i], bits_per_entry,
 				sbi->s_imap_size-i, bit, max_request);
 
 			if (run >= min_request)
