@@ -1,83 +1,28 @@
 /*
  * Optimized MPEG FS - inode and super operations.
  * Copyright (C) 2006 Bob Copeland <me@bobcopeland.com>
+ * Released under GPL v2.
  */
 #include <linux/version.h>
 #include <linux/module.h>
 #include <linux/sched.h>
 #include <linux/fs.h>
 #include <linux/vfs.h>
+#include <linux/parser.h>
 #include <linux/buffer_head.h>
 #include <linux/vmalloc.h>
+#include <linux/crc-itu-t.h>
 #include "omfs.h"
 
 MODULE_AUTHOR("Bob Copeland <me@bobcopeland.com>");
 MODULE_DESCRIPTION("OMFS (ReplayTV/Karma) Filesystem for Linux");
 MODULE_LICENSE("GPL");
 
-static struct kmem_cache *omfs_inode_cachep;
-
-static struct inode *omfs_alloc_inode(struct super_block *sb)
-{
-	struct omfs_inode_info *oinf;
-	oinf = (struct omfs_inode_info *) kmem_cache_alloc(omfs_inode_cachep,
-			GFP_KERNEL);
-	if (!oinf)
-		return NULL;
-	return &oinf->vfs_inode;
-}
-
-static void omfs_destroy_inode(struct inode *inode)
-{
-	kmem_cache_free(omfs_inode_cachep, OMFS_I(inode));
-}
-
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,23)
-static void init_once(void *p, struct kmem_cache *cachep, unsigned long flags)
-{
-#else
-static void init_once(struct kmem_cache *cachep, void *p)
-{
-#endif
-	struct omfs_inode_info *oinf = (struct omfs_inode_info *) p;
-
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,21)
-	if ((flags & (SLAB_CTOR_VERIFY|SLAB_CTOR_CONSTRUCTOR)) ==
-	    SLAB_CTOR_CONSTRUCTOR)
-#endif
-	inode_init_once(&oinf->vfs_inode);
-}
-
-static int init_inodecache(void)
-{
-	omfs_inode_cachep = kmem_cache_create("omfs_inode_cache",
-					     sizeof(struct omfs_inode_info),
-					     0, SLAB_RECLAIM_ACCOUNT | SLAB_MEM_SPREAD,
-					     init_once
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,22)
-                         , NULL
-#endif
-                         );
-	if (!omfs_inode_cachep)
-		return -ENOMEM;
-	return 0;
-}
-
-static void destroy_inodecache(void)
-{
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,18)
-	if (kmem_cache_destroy(omfs_inode_cachep))
-		printk(KERN_INFO "omfs_inode_cache: not all structures were freed\n");
-#else
-	kmem_cache_destroy(omfs_inode_cachep);
-#endif
-}
-
 struct inode *omfs_new_inode(struct inode *dir, int mode)
 {
 	struct inode *inode;
 	u64 new_block;
-	int res;
+	int err;
 	int len;
 	struct omfs_sb_info *sbi = OMFS_SB(dir->i_sb);
 
@@ -85,30 +30,25 @@ struct inode *omfs_new_inode(struct inode *dir, int mode)
 	if (!inode)
 		return ERR_PTR(-ENOMEM);
 
-	res = omfs_allocate_range(dir->i_sb, sbi->s_mirrors, sbi->s_mirrors,
+	err = omfs_allocate_range(dir->i_sb, sbi->s_mirrors, sbi->s_mirrors,
 			&new_block, &len);
-	if (res)
-		return ERR_PTR(res);
+	if (err)
+		goto fail;
 
 	inode->i_ino = new_block;
 	inode->i_mode = mode;
 	inode->i_uid = current->fsuid;
 	inode->i_gid = current->fsgid;
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,18)
-	inode->i_blksize = PAGE_SIZE;
-#endif
 	inode->i_blocks = 0;
 	inode->i_mapping->a_ops = &omfs_aops;
 
-	OMFS_I(inode)->i_state = OMFS_STATE_NEW;
-
 	inode->i_atime = inode->i_mtime = inode->i_ctime = CURRENT_TIME;
 	switch (mode & S_IFMT) {
 	case S_IFDIR:
 		inode->i_op = &omfs_dir_inops;
 		inode->i_fop = &omfs_dir_operations;
 		inode->i_size = sbi->s_sys_blocksize;
-		inode->i_nlink++;
+		inc_nlink(inode);
 		break;
 	case S_IFREG:
 		inode->i_op = &omfs_file_inops;
@@ -120,6 +60,34 @@ struct inode *omfs_new_inode(struct inode *dir, int mode)
 	insert_inode_hash(inode);
 	mark_inode_dirty(inode);
 	return inode;
+fail:
+	make_bad_inode(inode);
+	iput(inode);
+	return ERR_PTR(err);
+}
+
+/*
+ * Update the header checksums for a dirty inode based on its contents.
+ * Caller is expected to hold the buffer head underlying oi and mark it
+ * dirty.
+ */
+static void omfs_update_checksums(struct omfs_inode *oi)
+{
+	int xor, i, ofs = 0, count;
+	u16 crc = 0;
+	unsigned char *ptr = (unsigned char *) oi;
+
+	count = be32_to_cpu(oi->i_head.h_body_size);
+	ofs = sizeof(struct omfs_header);
+
+	crc = crc_itu_t(crc, ptr + ofs, count);
+	oi->i_head.h_crc = cpu_to_be16(crc);
+
+	xor = ptr[0];
+	for (i = 1; i < OMFS_XOR_COUNT; i++)
+		xor ^= ptr[i];
+
+	oi->i_head.h_check_xor = xor;
 }
 
 static int omfs_write_inode(struct inode *inode, int wait)
@@ -130,29 +98,27 @@ static int omfs_write_inode(struct inode *inode, int wait)
 	unsigned int block;
 	u64 ctime;
 	int i;
-	int ret = 0;
+	int ret = -EIO;
+	int sync_failed = 0;
 
 	/* get current inode since we may have written sibling ptrs etc. */
 	block = clus_to_blk(sbi, inode->i_ino);
 	bh = sb_bread(inode->i_sb, block);
-	if (!bh) {
-		ret = -EIO;
+	if (!bh)
 		goto out;
-	}
 
 	oi = (struct omfs_inode *) bh->b_data;
-	if (OMFS_I(inode)->i_state & OMFS_STATE_NEW)
-	{
-		OMFS_I(inode)->i_state &= ~OMFS_STATE_NEW;
-	}
 
 	oi->i_head.h_self = cpu_to_be64(inode->i_ino);
-	if (S_ISDIR(inode->i_mode)) {
+	if (S_ISDIR(inode->i_mode))
 		oi->i_type = OMFS_DIR;
-	} else if (S_ISREG(inode->i_mode)) {
+	else if (S_ISREG(inode->i_mode))
 		oi->i_type = OMFS_FILE;
-	} else
-		BUG();
+	else {
+		printk(KERN_WARNING "omfs: unknown file type: %d\n",
+			inode->i_mode);
+		goto out_brelse;
+	}
 
 	oi->i_head.h_body_size = cpu_to_be32(sbi->s_sys_blocksize -
 		sizeof(struct omfs_header));
@@ -165,44 +131,34 @@ static int omfs_write_inode(struct inode *inode, int wait)
 		((inode->i_ctime.tv_nsec + 999)/1000);
 	oi->i_ctime = cpu_to_be64(ctime);
 
-	if (omfs_update_checksums(oi, inode->i_sb, inode->i_ino) != 0) {
-		ret = -EIO;
-		goto out;
-	}
+	omfs_update_checksums(oi);
 
 	mark_buffer_dirty(bh);
-	if (wait)
-	{
+	if (wait) {
 		sync_dirty_buffer(bh);
 		if (buffer_req(bh) && !buffer_uptodate(bh))
-			ret = -EIO;
+			sync_failed = 1;
 	}
-	brelse(bh);
 
-	// if mirroring writes, copy to next fsblock
-	for (i = 0; i < sbi->s_mirrors; i++)
-	{
-		bh = sb_bread(inode->i_sb, block);
-		if (!bh)
-			return -EIO;
+	/* if mirroring writes, copy to next fsblock */
+	for (i = 1; i < sbi->s_mirrors; i++) {
 		bh2 = sb_bread(inode->i_sb, block + i *
 			(sbi->s_blocksize / sbi->s_sys_blocksize));
 		if (!bh2)
-		{
-			brelse(bh);
-			return -EIO;
-		}
+			goto out_brelse;
+
 		memcpy(bh2->b_data, bh->b_data, bh->b_size);
 		mark_buffer_dirty(bh2);
-		if (wait)
-		{
+		if (wait) {
 			sync_dirty_buffer(bh2);
 			if (buffer_req(bh2) && !buffer_uptodate(bh2))
-				ret = -EIO;
+				sync_failed = 1;
 		}
-		brelse(bh);
 		brelse(bh2);
 	}
+	ret = (sync_failed) ? -EIO : 0;
+out_brelse:
+	brelse(bh);
 out:
 	return ret;
 }
@@ -229,59 +185,35 @@ static void omfs_delete_inode(struct inode *inode)
 	clear_inode(inode);
 }
 
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,24)
-void omfs_read_inode(struct inode *inode)
-#else
 struct inode *omfs_iget(struct super_block *sb, ino_t ino)
-#endif
 {
+	struct omfs_sb_info *sbi = OMFS_SB(sb);
 	struct omfs_inode *oi;
-	struct omfs_inode_info *oinf;
 	struct buffer_head *bh;
 	unsigned int block;
 	u64 ctime;
 	unsigned long nsecs;
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,24)
-	ino_t ino = inode->i_ino;
-#else
 	struct inode *inode;
+
 	inode = iget_locked(sb, ino);
 	if (!inode)
 		return ERR_PTR(-ENOMEM);
 	if (!(inode->i_state & I_NEW))
 		return inode;
-#endif
 
- 	oinf = OMFS_I(inode);
-
-	// check against s_num_blocks
-	block = clus_to_blk(OMFS_SB(inode->i_sb), ino);
+	block = clus_to_blk(sbi, ino);
 	bh = sb_bread(inode->i_sb, block);
-	if (!bh) {
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,24)
-		make_bad_inode(inode);
-		return;
-#else
-		iget_failed(inode);
-		return ERR_PTR(-EIO);
-#endif
-	}
+	if (!bh)
+		goto iget_failed;
 
 	oi = (struct omfs_inode *)bh->b_data;
 
-	// check self
-	if (ino != be64_to_cpu(oi->i_head.h_self)) {
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,24)
-		make_bad_inode(inode);
-		return;
-#else
-		iget_failed(inode);
-		return ERR_PTR(-EIO);
-#endif
-	}
+	/* check self */
+	if (ino != be64_to_cpu(oi->i_head.h_self))
+		goto fail_bh;
 
-	inode->i_uid = 0;
-	inode->i_gid = 0;
+	inode->i_uid = sbi->s_uid;
+	inode->i_gid = sbi->s_gid;
 
 	ctime = be64_to_cpu(oi->i_ctime);
 	nsecs = do_div(ctime, 1000) * 1000L;
@@ -293,50 +225,44 @@ struct inode *omfs_iget(struct super_block *sb, ino_t ino)
 	inode->i_mtime.tv_nsec = nsecs;
 	inode->i_ctime.tv_nsec = nsecs;
 
-	oinf->i_state = 0;
 	inode->i_mapping->a_ops = &omfs_aops;
 
 	switch (oi->i_type) {
 	case OMFS_DIR:
-		inode->i_mode = S_IFDIR | S_IRUGO | S_IWUGO | S_IXUGO;
+		inode->i_mode = S_IFDIR | (S_IRWXUGO & ~sbi->s_dmask);
 		inode->i_op = &omfs_dir_inops;
 		inode->i_fop = &omfs_dir_operations;
 		inode->i_size = be32_to_cpu(oi->i_head.h_body_size) +
 			sizeof(struct omfs_header);
-		inode->i_nlink++;
+		inc_nlink(inode);
 		break;
 	case OMFS_FILE:
-		inode->i_mode = S_IFREG | S_IRUGO | S_IWUGO;
+		inode->i_mode = S_IFREG | (S_IRWXUGO & ~sbi->s_fmask);
 		inode->i_fop = &omfs_file_operations;
 		inode->i_size = be64_to_cpu(oi->i_size);
 		break;
 	}
 	brelse(bh);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,25)
 	unlock_new_inode(inode);
 	return inode;
-#endif
+fail_bh:
+	brelse(bh);
+iget_failed:
+	iget_failed(inode);
+	return ERR_PTR(-EIO);
 }
 
-
 static void omfs_put_super(struct super_block *sb)
 {
 	struct omfs_sb_info *sbi = OMFS_SB(sb);
-	if (sbi) {
-		kfree(sbi->s_imap);
-		kfree(sbi);
-	}
+	kfree(sbi->s_imap);
+	kfree(sbi);
 	sb->s_fs_info = NULL;
 }
 
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,17)
-static int omfs_statfs(struct super_block *s, struct kstatfs *buf)
-{
-#else
 static int omfs_statfs(struct dentry *dentry, struct kstatfs *buf)
 {
 	struct super_block *s = dentry->d_sb;
-#endif
 	struct omfs_sb_info *sbi = OMFS_SB(s);
 	buf->f_type = OMFS_MAGIC;
 	buf->f_bsize = sbi->s_blocksize;
@@ -349,16 +275,12 @@ static int omfs_statfs(struct dentry *dentry, struct kstatfs *buf)
 	return 0;
 }
 
-struct super_operations omfs_sops = {
-	.alloc_inode	= omfs_alloc_inode,
-	.destroy_inode	= omfs_destroy_inode,
+static struct super_operations omfs_sops = {
 	.write_inode	= omfs_write_inode,
 	.delete_inode	= omfs_delete_inode,
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,24)
-	.read_inode	= omfs_read_inode,
-#endif
 	.put_super	= omfs_put_super,
 	.statfs		= omfs_statfs,
+	.show_options	= generic_show_options,
 };
 
 /*
@@ -378,74 +300,149 @@ static int omfs_get_imap(struct super_block *sb)
 	unsigned long **ptr;
 	sector_t block;
 
-	bitmap_size = (sbi->s_num_blocks + 7) / 8;
-	array_size = (bitmap_size + sb->s_blocksize - 1) / sb->s_blocksize;
+	bitmap_size = DIV_ROUND_UP(sbi->s_num_blocks, 8);
+	array_size = DIV_ROUND_UP(bitmap_size, sb->s_blocksize);
+
+	if (sbi->s_bitmap_ino == ~0ULL)
+		goto out;
 
 	sbi->s_imap_size = array_size;
-	sbi->s_imap = kzalloc(array_size * sizeof(unsigned long), GFP_KERNEL);
+	sbi->s_imap = kzalloc(array_size * sizeof(unsigned long *), GFP_KERNEL);
 	if (!sbi->s_imap)
-		return -ENOMEM;
+		goto nomem;
 
 	block = clus_to_blk(sbi, sbi->s_bitmap_ino);
 	ptr = sbi->s_imap;
 	for (count = bitmap_size; count > 0; count -= sb->s_blocksize) {
 		bh = sb_bread(sb, block++);
 		if (!bh)
-			goto nomem;
+			goto nomem_free;
 		*ptr = kmalloc(sb->s_blocksize, GFP_KERNEL);
-		if (!*ptr)
-			goto nomem;
+		if (!*ptr) {
+			brelse(bh);
+			goto nomem_free;
+		}
 		memcpy(*ptr, bh->b_data, sb->s_blocksize);
+		if (count < sb->s_blocksize)
+			memset((void *)*ptr + count, 0xff,
+				sb->s_blocksize - count);
 		brelse(bh);
-		ptr ++;
+		ptr++;
 	}
+out:
 	return 0;
-nomem:
-	for (count=0; count<array_size; count++)
+
+nomem_free:
+	for (count = 0; count < array_size; count++)
 		kfree(sbi->s_imap[count]);
 
 	kfree(sbi->s_imap);
+nomem:
 	sbi->s_imap = NULL;
+	sbi->s_imap_size = 0;
 	return -ENOMEM;
 }
 
-static void set_block_shift(struct omfs_sb_info *sbi)
+enum {
+	Opt_uid, Opt_gid, Opt_umask, Opt_dmask, Opt_fmask
+};
+
+static match_table_t tokens = {
+	{Opt_uid, "uid=%u"},
+	{Opt_gid, "gid=%u"},
+	{Opt_umask, "umask=%o"},
+	{Opt_dmask, "dmask=%o"},
+	{Opt_fmask, "fmask=%o"},
+};
+
+static int parse_options(char *options, struct omfs_sb_info *sbi)
 {
-	unsigned int scale = sbi->s_blocksize / sbi->s_sys_blocksize;
-	sbi->s_block_shift = 0;
-	for (scale>>=1; scale; scale>>=1)
-		sbi->s_block_shift++;
+	char *p;
+	substring_t args[MAX_OPT_ARGS];
+	int option;
+
+	if (!options)
+		return 1;
+
+	while ((p = strsep(&options, ",")) != NULL) {
+		int token;
+		if (!*p)
+			continue;
+
+		token = match_token(p, tokens, args);
+		switch (token) {
+		case Opt_uid:
+			if (match_int(&args[0], &option))
+				return 0;
+			sbi->s_uid = option;
+			break;
+		case Opt_gid:
+			if (match_int(&args[0], &option))
+				return 0;
+			sbi->s_gid = option;
+			break;
+		case Opt_umask:
+			if (match_octal(&args[0], &option))
+				return 0;
+			sbi->s_fmask = sbi->s_dmask = option;
+			break;
+		case Opt_dmask:
+			if (match_octal(&args[0], &option))
+				return 0;
+			sbi->s_dmask = option;
+			break;
+		case Opt_fmask:
+			if (match_octal(&args[0], &option))
+				return 0;
+			sbi->s_fmask = option;
+			break;
+		default:
+			return 0;
+		}
+	}
+	return 1;
 }
 
 static int omfs_fill_super(struct super_block *sb, void *data, int silent)
 {
-	struct buffer_head *bh=NULL, *bh2=NULL;
+	struct buffer_head *bh, *bh2;
 	struct omfs_super_block *omfs_sb;
 	struct omfs_root_block *omfs_rb;
 	struct omfs_sb_info *sbi;
 	struct inode *root;
 	sector_t start;
-	int ret = 0;
+	int ret = -EINVAL;
+
+	save_mount_options(sb, (char *) data);
 
 	sbi = kzalloc(sizeof(struct omfs_sb_info), GFP_KERNEL);
 	if (!sbi)
 		return -ENOMEM;
+
 	sb->s_fs_info = sbi;
 
+	sbi->s_uid = current->uid;
+	sbi->s_gid = current->gid;
+	sbi->s_dmask = sbi->s_fmask = current->fs->umask;
+
+	if (!parse_options((char *) data, sbi))
+		goto end;
+
 	sb->s_maxbytes = 0xffffffff;
 
 	sb_set_blocksize(sb, 0x200);
 
-	if (!(bh = sb_bread(sb, 0)))
-		goto out;
+	bh = sb_bread(sb, 0);
+	if (!bh)
+		goto end;
 
 	omfs_sb = (struct omfs_super_block *)bh->b_data;
 
-	if (be32_to_cpu(omfs_sb->s_magic) != OMFS_MAGIC) {
+	if (omfs_sb->s_magic != cpu_to_be32(OMFS_MAGIC)) {
 		if (!silent)
 			printk(KERN_ERR "omfs: Invalid superblock (%x)\n",
 				   omfs_sb->s_magic);
-		goto out;
+		goto out_brelse_bh;
 	}
 	sb->s_magic = OMFS_MAGIC;
 
@@ -456,73 +453,83 @@ static int omfs_fill_super(struct super_block *sb, void *data, int silent)
 	sbi->s_sys_blocksize = be32_to_cpu(omfs_sb->s_sys_blocksize);
 	mutex_init(&sbi->s_bitmap_lock);
 
-	// Use sys_blocksize as the fs block since it is smaller than a
-	// page while the fs blocksize can be larger.
+	if (sbi->s_sys_blocksize > PAGE_SIZE) {
+		printk(KERN_ERR "omfs: sysblock size (%d) is out of range\n",
+			sbi->s_sys_blocksize);
+		goto out_brelse_bh;
+	}
+
+	if (sbi->s_blocksize < sbi->s_sys_blocksize ||
+	    sbi->s_blocksize > OMFS_MAX_BLOCK_SIZE) {
+		printk(KERN_ERR "omfs: block size (%d) is out of range\n",
+			sbi->s_blocksize);
+		goto out_brelse_bh;
+	}
+
+	/*
+	 * Use sys_blocksize as the fs block since it is smaller than a
+	 * page while the fs blocksize can be larger.
+	 */
 	sb_set_blocksize(sb, sbi->s_sys_blocksize);
 
-	// and the difference goes into a shift.  sys_blocksize is always
-	// a power of two factor of blocksize.
-	set_block_shift(sbi);
+	/*
+	 * ...and the difference goes into a shift.  sys_blocksize is always
+	 * a power of two factor of blocksize.
+	 */
+	sbi->s_block_shift = get_bitmask_order(sbi->s_blocksize) -
+		get_bitmask_order(sbi->s_sys_blocksize);
 
 	start = clus_to_blk(sbi, be64_to_cpu(omfs_sb->s_root_block));
-	if (!(bh2 = sb_bread(sb, start)))
-		goto out;
+	bh2 = sb_bread(sb, start);
+	if (!bh2)
+		goto out_brelse_bh;
 
 	omfs_rb = (struct omfs_root_block *)bh2->b_data;
 
 	sbi->s_bitmap_ino = be64_to_cpu(omfs_rb->r_bitmap);
 	sbi->s_clustersize = be32_to_cpu(omfs_rb->r_clustersize);
 
+	if (sbi->s_num_blocks != be64_to_cpu(omfs_rb->r_num_blocks)) {
+		printk(KERN_ERR "omfs: block count discrepancy between "
+			"super and root blocks (%llx, %llx)\n",
+			sbi->s_num_blocks, be64_to_cpu(omfs_rb->r_num_blocks));
+		goto out_brelse_bh2;
+	}
+
 	ret = omfs_get_imap(sb);
 	if (ret)
-		goto end;
+		goto out_brelse_bh2;
 
 	sb->s_op = &omfs_sops;
 
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,24)
-	root = iget(sb, be64_to_cpu(omfs_rb->r_root_dir));
-#else
 	root = omfs_iget(sb, be64_to_cpu(omfs_rb->r_root_dir));
 	if (IS_ERR(root)) {
 		ret = PTR_ERR(root);
-		goto out;
-	}	
-#endif
+		goto out_brelse_bh2;
+	}
 
 	sb->s_root = d_alloc_root(root);
 	if (!sb->s_root) {
 		iput(root);
-		goto out;
+		goto out_brelse_bh2;
 	}
 	printk(KERN_DEBUG "omfs: Mounted volume %s\n", omfs_rb->r_name);
-	goto end;
-
-out:
-	ret = -EINVAL;
 
+	ret = 0;
+out_brelse_bh2:
+	brelse(bh2);
+out_brelse_bh:
+	brelse(bh);
 end:
-	if (bh2)
-		brelse(bh2);
-	if (bh)
-		brelse(bh);
 	return ret;
 }
 
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,17)
-static struct super_block *omfs_get_sb(struct file_system_type *fs_type,
-					   int flags, const char *dev_name,
-					   void *data)
-{
-	return get_sb_bdev(fs_type, flags, dev_name, data, omfs_fill_super);
-}
-#else
 static int omfs_get_sb(struct file_system_type *fs_type,
 			int flags, const char *dev_name,
 			void *data, struct vfsmount *m)
 {
 	return get_sb_bdev(fs_type, flags, dev_name, data, omfs_fill_super, m);
 }
-#endif
 
 static struct file_system_type omfs_fs_type = {
 	.owner = THIS_MODULE,
@@ -534,21 +541,12 @@ static struct file_system_type omfs_fs_type = {
 
 static int __init init_omfs_fs(void)
 {
-	int err = init_inodecache();
-	if (err)
-		goto out;
-
-	err = register_filesystem(&omfs_fs_type);
-	if (err)
-		destroy_inodecache();
-out:
-	return err;
+	return register_filesystem(&omfs_fs_type);
 }
 
 static void __exit exit_omfs_fs(void)
 {
 	unregister_filesystem(&omfs_fs_type);
-	destroy_inodecache();
 }
 
 module_init(init_omfs_fs);
